@@ -59,8 +59,6 @@ import univention.admin.syntax as udm_syntax
 import univention.admin.uexceptions as udm_errors
 import univention.admin.mapping as udm_mapping
 
-from univention.management.console.modules.udm.syntax import widget, default_value
-
 from ldap import LDAPError, NO_SUCH_OBJECT
 from ldap.filter import filter_format
 from ldap.dn import explode_dn
@@ -459,8 +457,8 @@ class UDM_Module(object):
 		ldap_connection, ldap_position = self.get_ldap_connection()
 		for key, prop in getattr(self.module, 'property_descriptions', {}).items():
 			if key == property_name:
-				value = default_value(prop.syntax)
-				if isinstance(value, (list, tuple)):
+				value = prop.syntax.widget_default_search_pattern
+				if prop.syntax.search_widget in ('ComboBox', 'SuggestionBox'):
 					value = read_syntax_choices(prop.syntax, ldap_connection=ldap_connection, ldap_position=ldap_position)
 				return value
 
@@ -542,12 +540,12 @@ class UDM_Module(object):
 			ldap_position.setDn(container)
 
 		if superordinate:
-			mod = get_module(self.name, superordinate, ldap_connection)
+			_superordinate, mod = get_obj_module(self.name, superordinate, ldap_connection)
 			if not mod:
 				MODULE.error('Superordinate module not found: %s' % (superordinate,))
 				raise SuperordinateDoesNotExist(superordinate)
 			MODULE.info('Found UDM module for superordinate')
-			superordinate = mod.get(superordinate)
+			superordinate = _superordinate
 
 		obj = self.module.object(None, ldap_connection, ldap_position, superordinate=superordinate)
 		try:
@@ -651,7 +649,7 @@ class UDM_Module(object):
 			MODULE.warn('Failed to modify LDAP object %s: %s: %s' % (obj.dn, e.__class__.__name__, str(e)))
 			UDM_Error(e).reraise()
 
-	def search(self, container=None, attribute=None, value=None, superordinate=None, scope='sub', filter='', simple=False, simple_attrs=None, hidden=True, serverctrls=None, response=None):
+	def search(self, container=None, attribute=None, value=None, superordinate=None, scope='sub', filter='', simple=False, simple_attrs=None, hidden=True, serverctrls=None, response=None, allow_asterisks=True):
 		"""Searches for LDAP objects based on a search pattern"""
 		ldap_connection, ldap_position = self.get_ldap_connection()
 		if container == 'all':
@@ -661,7 +659,7 @@ class UDM_Module(object):
 		if attribute in [None, 'None'] and filter:
 			filter_s = str(filter)
 		else:
-			filter_s = _object_property_filter(self, attribute, value, hidden)
+			filter_s = self._object_property_filter(attribute, value, hidden, allow_asterisks)
 
 		MODULE.info('Searching for LDAP objects: container = %s, filter = %s, superordinate = %s' % (container, filter_s, superordinate))
 		result = None
@@ -844,11 +842,10 @@ class UDM_Module(object):
 
 	@property
 	def default_search_attrs(self):
-		ret = []
-		for key, prop in getattr(self.module, 'property_descriptions', {}).items():
-			if prop.include_in_default_search:
-				ret.append(key)
-		return ret
+		return [
+			key for key, prop in self.module.property_descriptions.items()
+			if prop.include_in_default_search
+		]
 
 	def obj_description(self, obj):
 		description = None
@@ -946,6 +943,8 @@ class UDM_Module(object):
 				'options': copy.deepcopy(prop.options),
 				'readonly': not bool(prop.editable),
 				'searchable': not prop.dontsearch,
+				'default_search_pattern': prop.syntax.widget_default_search_pattern,
+				'search_widget': prop.syntax.search_widget,
 				'multivalue': bool(prop.multivalue),
 				'identifies': bool(prop.identifies),
 				'threshold': prop.threshold,
@@ -980,7 +979,7 @@ class UDM_Module(object):
 					item['default'] = default_group
 
 			# read UCR configuration
-			item.update(widget(prop.syntax, item))
+			item.update(prop.syntax.get_widget_options(prop))
 
 			if prop.nonempty_is_default and 'default' not in item:
 				# Some properties have an empty value as first item.
@@ -1091,10 +1090,23 @@ class UDM_Module(object):
 
 	def get_policy_references(self, dn):
 		ldap_connection, ldap_position = self.get_ldap_connection()
+		references = []
 		if self.is_policy_module():  # TODO: move into the handlers/policies/*.py
 			search_filter = filter_format("(&(objectClass=univentionPolicyReference)(univentionPolicyReference=%s))", (dn,))
-			return read_syntax_choices(udm_syntax.LDAP_Search(filter=search_filter, viewonly=True), ldap_connection=ldap_connection, ldap_position=ldap_position)
-		return []
+			for dn in ldap_connection.searchDn(filter=search_filter):
+				obj, module = get_obj_module(None, dn, ldap_connection)
+				if not module or not obj:
+					continue
+				label = '%s: %s' % (module.title, obj.description())
+				references.append({
+					'module': 'udm',
+					'flavor': module.flavor or 'navigation',
+					'objectType': module.name,
+					'id': dn,
+					'label': label,
+					'icon': 'udm-%s' % module.name.replace('/', '-')
+				})
+		return references
 
 	def get_references(self, obj):
 		references = []
@@ -1136,6 +1148,30 @@ class UDM_Module(object):
 			return self.module.mapping
 		return udm_mapping.mapping()
 
+	def _object_property_filter(self, object_property, object_property_value, show_hidden=True, allow_asterisks=True):
+		if object_property in [None, 'None']:
+			ret = ''
+			if object_property_value not in [None, '*']:
+				ret = '(|%s)' % ''.join(udm_syntax.ISyntax.get_object_property_filter(attr, object_property_value, allow_asterisks) for attr in self.default_search_attrs)
+		else:
+			prop = self.module.property_descriptions.get(object_property)
+			syn = prop.syntax if prop else udm_syntax.ISyntax
+			ret = syn.get_object_property_filter(object_property, object_property_value, allow_asterisks=allow_asterisks)
+
+		if not show_hidden:
+			ret = self._append_hidden_filter(ret)
+		return ret
+
+	def _append_hidden_filter(self, ret):
+		if self.module is None or self.module.property_descriptions.get('objectFlag') is None:
+			return ret
+		hidden_filter = '!(objectFlag=hidden)'
+		if ret:
+			if not ret.startswith('('):
+				ret = '(%s)' % ret
+			return '(&(%s)%s)' % (hidden_filter, ret)
+		return hidden_filter
+
 
 def container_modules():
 	containers = []
@@ -1172,11 +1208,21 @@ def ldap_dn2path(ldap_dn, include_rdn=True):
 
 def get_module(flavor, ldap_dn, ldap_connection=None, ldap_position=None):
 	"""Determines an UDM module handling the LDAP object identified by the given LDAP DN"""
+	return _get_module(flavor, ldap_dn, None, ldap_connection, ldap_position)
+
+
+def get_obj_module(flavor, ldap_dn, ldap_connection=None, ldap_position=None):
+	attr = ldap_connection.get(ldap_dn)
+	module = _get_module(flavor, ldap_dn, attr, ldap_connection, ldap_position)
+	return module.get(ldap_dn, attributes=attr), module
+
+
+def _get_module(flavor, ldap_dn, attributes=None, ldap_connection=None, ldap_position=None):
 	if flavor is None or flavor == 'navigation':
 		base = None
 	else:
 		base, name = split_module_name(flavor)
-	modules = udm_modules.objectType(None, ldap_connection, ldap_dn, module_base=base)
+	modules = udm_modules.objectType(None, ldap_connection, ldap_dn, attributes, module_base=base)
 
 	if not modules:
 		return None
@@ -1231,64 +1277,6 @@ def list_objects(container, object_type=None, ldap_connection=None, ldap_positio
 				MODULE.error('Could not load object %r (%r) exception: %s' % (dn, module.module, traceback.format_exc()))
 
 
-def split_module_attr(value):
-	if ': ' in value:
-		return value.split(': ', 1)
-	return (None, value)
-
-
-def _object_property_filter(module, object_property, object_property_value, show_hidden=True):
-	if object_property in [None, 'None'] and module is not None:
-		default_search_attrs = module.default_search_attrs
-		if default_search_attrs and object_property_value not in [None, '*']:
-			ret = '(|%s)' % ''.join('(%s=%s)' % (attr, object_property_value) for attr in default_search_attrs)
-		else:
-			ret = ''
-	else:
-		ret = ''
-		prop = module.module.property_descriptions.get(object_property)
-		if prop and issubclass(prop.syntax if inspect.isclass(prop.syntax) else type(prop.syntax), (udm_syntax.IStates, udm_syntax.boolean)):
-			ret = prop.syntax.get_object_property_filter(object_property, object_property_value)
-		if not ret:
-			ret = '%s=%s' % (object_property, object_property_value)
-			no_substring_value = object_property_value.strip('*')
-			if no_substring_value and no_substring_value != object_property_value:
-				ret = '(|(%s)(%s=%s))' % (ret, object_property, no_substring_value)
-	if module is not None:
-		hidden_flag_attribute = 'objectFlag'
-		has_hidden_flag = module.get_property(hidden_flag_attribute) is not None
-		if has_hidden_flag and not show_hidden:
-			if ret:
-				if not ret.startswith('('):
-					ret = '(%s)' % ret
-				ret = '(&(!(%s=hidden))%s)' % (hidden_flag_attribute, ret)
-			else:
-				ret = '!(%s=hidden)' % hidden_flag_attribute
-	return ret
-
-
-def _create_ldap_filter(syn, options, module=None):
-	if syn.depends and syn.depends not in options:
-		return None
-	if callable(syn.udm_filter):
-		filter_s = syn.udm_filter(options)
-	else:
-		filter_s = syn.udm_filter % options
-	if options.get('objectProperty') and options.get('objectPropertyValue'):
-		if filter_s and not filter_s.startswith('('):
-			# make sure that the LDAP filter is wrapped in brackets
-			filter_s = '(%s)' % filter_s
-		object_property = options.get('objectProperty')
-		object_property_value = options.get('objectPropertyValue')
-		show_hidden = options.get('hidden', True)
-		property_filter_s = _object_property_filter(module, object_property, object_property_value, show_hidden)
-		if property_filter_s and not property_filter_s.startswith('('):
-			# make sure that the LDAP filter is wrapped in brackets
-			property_filter_s = '(%s)' % property_filter_s
-		filter_s = '(&%s%s)' % (property_filter_s, filter_s)
-	return filter_s
-
-
 LDAP_ATTR_RE = re.compile(r'^%\(([^)]*)\)s$')  # '%(username)s' -> 'username'
 
 
@@ -1301,9 +1289,8 @@ def _get_syntax(syntax_name):
 def search_syntax_choices_by_key(syn, key, ldap_connection, ldap_position):
 	if issubclass(syn.__class__, udm_syntax.UDM_Objects):
 		if syn.key == 'dn':
-			module_search_options = {'scope': 'base', 'container': key}
 			try:
-				return read_syntax_choices(syn, {}, module_search_options, ldap_connection=ldap_connection, ldap_position=ldap_position)
+				return read_syntax_choices(syn, {'scope': 'base', 'container': key}, ldap_connection=ldap_connection, ldap_position=ldap_position)
 			except udm_errors.base:  # TODO: which exception is raised here exactly?
 				# invalid DN
 				return []
@@ -1311,7 +1298,7 @@ def search_syntax_choices_by_key(syn, key, ldap_connection, ldap_position):
 			match = LDAP_ATTR_RE.match(syn.key)
 			if match:
 				attr = match.groups()[0]
-				options = {'objectProperty': attr, 'objectPropertyValue': key}
+				options = {'objectProperty': attr, 'objectPropertyValue': key, 'allow_asterisks': False}
 				return read_syntax_choices(syn, options, ldap_connection=ldap_connection, ldap_position=ldap_position)
 
 	MODULE.warn('Syntax %r: No fast search function' % syn.name)
@@ -1320,7 +1307,7 @@ def search_syntax_choices_by_key(syn, key, ldap_connection, ldap_position):
 	return read_syntax_choices(syn, ldap_connection=ldap_connection, ldap_position=ldap_position)
 
 
-def info_syntax_choices(syn, options={}, ldap_connection=None, ldap_position=None):
+def info_syntax_choices(syn, options=None, ldap_connection=None, ldap_position=None):
 	if issubclass(syn.__class__, udm_syntax.UDM_Objects):
 		size = 0
 		if syn.static_values is not None:
@@ -1329,7 +1316,7 @@ def info_syntax_choices(syn, options={}, ldap_connection=None, ldap_position=Non
 			module = UDM_Module(udm_module, ldap_connection=ldap_connection, ldap_position=ldap_position)
 			if module.module is None:
 				continue
-			filter_s = _create_ldap_filter(syn, options, module)
+			filter_s = syn._create_ldap_filter(options or {}, module)
 			if filter_s is not None:
 				try:
 					size += len(module.search(filter=filter_s, simple=not syn.use_objects))
@@ -1339,252 +1326,50 @@ def info_syntax_choices(syn, options={}, ldap_connection=None, ldap_position=Non
 	return {'size': 0, 'performs_well': False}
 
 
-def read_syntax_choices(syn, options={}, module_search_options={}, ldap_connection=None, ldap_position=None):
+def read_syntax_choices(syn, options=None, ldap_connection=None, ldap_position=None):
 	syn = syn() if inspect.isclass(syn) else syn
-	syntax_name = syn.name
 
-	choices = getattr(syn, 'choices', [])
+	options = options or {}
+	options.setdefault('dependencies', {})
+	options['sizelimit'] = int(ucr.get('directory/manager/web/sizelimit', '2000') or 2000)
+	if '$dn$' in options:
+		options['dn'] = options.pop('$dn$')
 
-	if issubclass(syn.__class__, udm_syntax.UDM_Objects):
-		choices = []
-		# try to avoid using the slow udm interface
-		simple = False
-		attr = set()
-		if not syn.use_objects:
-			attr.update(re.findall(r'%\(([^)]+)\)', syn.key))
-			if syn.label:
-				attr.update(re.findall(r'%\(([^)]+)\)', syn.label))
-			for udm_module in syn.udm_modules:
-				module = UDM_Module(udm_module, ldap_connection=ldap_connection, ldap_position=ldap_position)
-				if not module.allows_simple_lookup():
-					break
-				if module is not None:
-					mapping = module.module.mapping
-					if not all([mapping.mapName(att) for att in attr]):
-						break
-			else:
-				simple = True
-			if not simple:
-				MODULE.warn('Syntax %s wants to get optimizations but may not. This is a Bug! We provide a fallback but the syntax will respond much slower than it could!' % syntax_name)
+	if 'container' in options:
+		options['base'] = options.pop('container') if options['container'] != 'all' else None
+	if 'objectProperty' in options:
+		options['property'] = options.pop('objectProperty') if options['objectProperty'] != 'None' else None
+	if 'objectPropertyValue' in options:
+		options['value'] = options.pop('objectPropertyValue')
+	# only for syntax_choices_key
+	options['allow_asterisks'] = options.pop('allow_asterisks', True)
 
-		def extract_key_label(syn, dn, info):
-			key = label = None
-			if syn.key == 'dn':
-				key = dn
-			else:
-				try:
-					key = syn.key % info
-				except KeyError:
-					pass
-			if syn.label == 'dn':
-				label = dn
-			elif syn.label is None:
-				pass
-			else:
-				try:
-					label = syn.label % info
-				except KeyError:
-					pass
-			return key, label
-		if not simple:
-			def map_choices(obj_list):
-				result = []
-				for obj in obj_list:
-					# first try it without obj.open() (expensive)
-					key, label = extract_key_label(syn, obj.dn, obj.info)
-					if key is None or label is None:
-						obj.open()
-						key, label = extract_key_label(syn, obj.dn, obj.info)
-						if key is None:
-							# ignore the entry as the key is important for a selection, there
-							# is no sensible fallback for the key (Bug #26994)
-							continue
-						if label is None:
-							# fallback to the default description as this is just what displayed
-							# to the user (Bug #26994)
-							label = udm_objects.description(obj)
-					result.append((key, label))
-				return result
-
-			for udm_module in syn.udm_modules:
-				module = UDM_Module(udm_module, ldap_connection=ldap_connection, ldap_position=ldap_position)
-				if module.module is None:
-					continue
-				filter_s = _create_ldap_filter(syn, options, module)
-				if filter_s is not None:
-					search_options = {'filter': filter_s}
-					search_options.update(module_search_options)
-					choices.extend(map_choices(module.search(**search_options)))
+	try:
+		if isinstance(syn, udm_syntax.LDAP_Search):
+			choices = []
+			for choice in syn.get_umc_choices(ldap_connection, options):
+				module = UDM_Module(choice['objectType'], ldap_connection=ldap_connection, ldap_position=ldap_position)
+				choice.update({
+					'module': 'udm',
+					'flavor': module.flavor,
+					'icon': 'udm-%s' % module.name.replace('/', '-'),
+				})
+				choices.append(choice)
 		else:
-			for udm_module in syn.udm_modules:
-				module = UDM_Module(udm_module, ldap_connection=ldap_connection, ldap_position=ldap_position)
-				if module.module is None:
-					continue
-				filter_s = _create_ldap_filter(syn, options, module)
-				if filter_s is not None:
-					if filter_s and not filter_s.startswith('('):
-						filter_s = '(%s)' % filter_s
-					mapping = module.module.mapping
-					ldap_attr = [mapping.mapName(att) for att in attr]
-					search_options = {'filter': filter_s, 'simple': True}
-					search_options.update(module_search_options)
-					if ldap_attr:
-						search_options['simple_attrs'] = ldap_attr
-						result = module.search(**search_options)
-						for dn, ldap_map in result:
-							info = udm_mapping.mapDict(mapping, ldap_map)
-							key, label = extract_key_label(syn, dn, info)
-							if key is None:
-								continue
-							if label is None:
-								label = ldap_connection.explodeDn(dn, 1)[0]
-							choices.append((key, label))
-					else:
-						keys = module.search(**search_options)
-						if syn.label == 'dn':
-							labels = keys
-						else:
-							labels = [ldap_connection.explodeDn(dn, 1)[0] for dn in keys]
-						choices.extend(zip(keys, labels))
-	elif issubclass(syn.__class__, udm_syntax.UDM_Attribute):
-		choices = []
-
-		def filter_choice(obj):
-			# if attributes does not exist or is empty
-			return syn.attribute in obj.info and obj.info[syn.attribute]
-
-		def map_choice(obj):
-			obj.open()
-			MODULE.info('Loading choices from %s: %s' % (obj.dn, obj.info))
-			try:
-				values = obj.info[syn.attribute]
-			except KeyError:
-				MODULE.warn('Object has no attribute %r' % (syn.attribute,))
-				# this happens for example in PrinterDriverList
-				# if the ldap schema is not installed
-				# and thus no 'printmodel' attribute is known.
-				return []
-			if not isinstance(values, (list, tuple)):  # single value
-				values = [values]
-			if syn.is_complex:
-				return [(x[syn.key_index], x[syn.label_index]) for x in values]
-			if syn.label_format is not None:
-				_choices = []
-				for value in values:
-					obj.info['$attribute$'] = value
-					_choices.append((value, syn.label_format % obj.info))
-				return _choices
-			return [(x, x) for x in values]
-
-		module = UDM_Module(syn.udm_module, ldap_connection=ldap_connection, ldap_position=ldap_position)
-		if module.module is None:
-			return []
-		MODULE.info('Found syntax %s with udm_module property' % syntax_name)
-		if syn.udm_filter == 'dn':
-			choices = map_choice(module.get(options[syn.depends]))
-		else:
-			filter_s = _create_ldap_filter(syn, options, module)
-			if filter_s is not None:
-				for element in map(map_choice, filter(filter_choice, module.search(filter=filter_s))):
-					for item in element:
-						choices.append(item)
-	elif issubclass(syn.__class__, udm_syntax.ldapDn) and hasattr(syn, 'searchFilter'):
-		try:
-			result = ldap_connection.searchDn(filter=syn.searchFilter)
-		except udm_errors.base:
-			MODULE.process('Failed to initialize syntax class %s' % syntax_name)
-			return []
-		choices = []
-		for dn in result:
-			dn_list = ldap_connection.explodeDn(dn)
-			choices.append((dn, dn_list[0].split('=', 1)[1]))
-
-	choices = [{'id': x[0], 'label': x[1]} for x in choices]
-
-	if issubclass(syn.__class__, udm_syntax.LDAP_Search):
-		options = options.get('options', {})
-		try:
-			syntax = udm_syntax.LDAP_Search(options['syntax'], options['filter'], options['attributes'], options['base'], options['value'], options['viewonly'], options['empty'], options['empty_end'])
-		except KeyError:
-			syntax = syn
-
-		if '$dn$' in options:
-			filter_mod = get_module(None, options['$dn$'], ldap_connection)
-			if filter_mod:
-				obj = filter_mod.get(options['$dn$'])
-				syntax.filter = udm.pattern_replace(syntax.filter, obj)
-
-		syntax._prepare(ldap_connection, syntax.filter)
-
-		choices = []
-		for item in syntax.values:
-			if syntax.viewonly:
-				dn, display_attr = item
-			else:
-				dn, store_pattern, display_attr = item
-
-			if display_attr:
-				# currently we just support one display attribute
-				mod_display, display = split_module_attr(display_attr[0])
-				module = get_module(mod_display, dn, ldap_connection)  # mod_display might be None
-			else:
-				module = get_module(None, dn, ldap_connection)
-				display = None
-			if not module:
-				continue
-			obj = module.get(dn)
-			if not obj:
-				continue
-
-			# find the value to store
-			if not syntax.viewonly:
-				mod_store, store = split_module_attr(store_pattern)
-				if store == 'dn':
-					id = dn
-				elif store in obj:
-					id = obj[store]
-				elif store in obj.oldattr and obj.oldattr[store]:
-					id = obj.oldattr[store][0].decode(*module.mapping.getEncoding(store))
-				else:
-					# no valid store object, ignore
-					MODULE.warn('LDAP_Search syntax %r: %r is no valid property for object %r - ignoring entry.' % (syntax.name, store, dn))
-					continue
-
-			# find the value to display
-			if display == 'dn':
-				label = dn
-			elif display is None:  # if view-only and in case of error
-				label = '%s: %s' % (module.title, obj[module.identifies])
-			else:
-				if display in obj:
-					label = obj[display]
-				elif display in obj.oldattr and obj.oldattr[display]:
-					label = obj.oldattr[display][0].decode(*module.module.mapping.getEncoding(display))
-				else:
-					MODULE.warn('LDAP_Search syntax %r: defines unknown attribute %r' % (syntax.name, display))
-					label = 'Unknown attribute %r' % (display,)
-
-			# create list entry
-			if syntax.viewonly:
-				choices.append({'module': 'udm', 'flavor': module.flavor or 'navigation', 'objectType': module.name, 'id': dn, 'label': label, 'icon': 'udm-%s' % module.name.replace('/', '-')})
-			else:
-				choices.append({'module': 'udm', 'flavor': module.flavor or 'navigation', 'objectType': module.name, 'id': id, 'label': label, 'icon': 'udm-%s' % module.name.replace('/', '-')})
-
-	# sort choices before inserting / appending some special items
-	choices = sorted(choices, key=lambda choice: choice['label'])
-
-	if issubclass(syn.__class__, (udm_syntax.UDM_Objects, udm_syntax.UDM_Attribute)):
-		if isinstance(syn.static_values, (tuple, list)):
-			for value in syn.static_values:
-				choices.insert(0, {'id': value[0], 'label': value[1]})
-		if syn.empty_value:
-			choices.insert(0, {'id': '', 'label': ''})
-	elif issubclass(syn.__class__, udm_syntax.LDAP_Search):
-		# then append empty value
-		if syntax.addEmptyValue:
-			choices.insert(0, {'id': '', 'label': ''})
-		elif syntax.appendEmptyValue:
-			choices.append({'id': '', 'label': ''})
+			choices = syn.get_choices(ldap_connection, options)
+			choices = [{'id': x[0], 'label': x[1]} for x in choices]
+	except udm_errors.ldapTimeout:
+		raise SearchTimeoutError()
+	except udm_errors.ldapSizelimitExceeded:
+		raise SearchLimitReached()
+	except (LDAPError, udm_errors.ldapError):
+		raise
+	except udm_errors.base as e:
+		if isinstance(e, udm_errors.noObject):
+			container = options.get('base')
+			if container and not ldap_connection.get(container):
+				raise ObjectDoesNotExist(container)
+		UDM_Error(e).reraise()
 
 	return choices
 
